#
# live.py : LiveImageCreator class for creating Live CD images
#
# Copyright 2007-2012, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys
import os
import os.path
import glob
import shutil
import subprocess
import logging
import re
import commands

from imgcreate.errors import *
from imgcreate.fs import *
from imgcreate.creator import *

class LiveImageCreatorBase(LoopImageCreator):
    """A base class for LiveCD image creators.

    This class serves as a base class for the architecture-specific LiveCD
    image creator subclass, LiveImageCreator.

    LiveImageCreator creates a bootable ISO containing the system image,
    bootloader, bootloader configuration, kernel and initramfs.

    """

    def __init__(self, ks, name, fslabel=None, releasever=None, tmpdir="/tmp",
                 title="Linux", product="Linux", useplugins=False, cacheonly=False,
                 docleanup=True, arch=None, runopts=None):
        """Initialise a LiveImageCreator instance.

        This method takes the same arguments as LoopImageCreator.__init__().

        """
        LoopImageCreator.__init__(self, ks, name,
                                  fslabel=fslabel,
                                  releasever=releasever,
                                  tmpdir=tmpdir,
                                  useplugins=useplugins,
                                  cacheonly=cacheonly,
                                  docleanup=docleanup)

        self.compress_type = "xz"
        """mksquashfs compressor to use."""

        self.skip_compression = False
        """Controls whether to use squashfs to compress the image."""

        self.skip_minimize = False
        """Controls whether an image minimizing snapshot should be created.

        This snapshot can be used when copying the system image from the ISO in
        order to minimize the amount of data that needs to be copied; simply,
        it makes it possible to create a version of the image's filesystem with
        no spare space.

        """

        self._timeout = kickstart.get_timeout(self.ks, 10)
        """The bootloader timeout from kickstart."""

        self._default_kernel = kickstart.get_default_kernel(self.ks, "kernel")
        """The default kernel type from kickstart."""

        self.__isodir = None
        self.__imagedir = None

        self.__modules = ["=ata", "sym53c8xx", "aic7xxx", "=usb", "=firewire",
                          "=mmc", "=pcmcia", "mptsas", "udf", "virtio_blk",
                          "virtio_pci", "virtio_scsi", "virtio_net", "virtio_mmio",
                          "virtio_balloon", "virtio-rng"]
        self.__modules.extend(kickstart.get_modules(self.ks))

        self._isofstype = "iso9660"
        self.base_on = False

        self.title = title
        self.product = product

        self.arch=arch
        self.run_options = runopts

    #
    # Hooks for subclasses
    #
    def _configure_bootloader(self, isodir):
        """Create the architecture specific booloader configuration.

        This is the hook where subclasses must create the booloader
        configuration in order to allow a bootable ISO to be built.

        isodir -- the directory where the contents of the ISO are to be staged

        """
        raise CreatorError("Bootloader configuration is arch-specific, "
                           "but not implemented for this arch!")

    def _get_kernel_options(self):
        """Return a kernel options string for bootloader configuration.

        This is the hook where subclasses may specify a set of kernel options
        which should be included in the images bootloader configuration.

        A sensible default implementation is provided.

        """
        r = kickstart.get_kernel_args(self.ks)
        if os.path.exists(self._instroot + "/usr/bin/rhgb"):
            r += " rhgb"
        if os.path.exists(self._instroot + "/usr/bin/plymouth"):
            r += " rhgb"
        return r

    def _get_mkisofs_options(self, isodir):
        """Return the architecture specific mkisosfs options.

        This is the hook where subclasses may specify additional arguments to
        mkisofs, e.g. to enable a bootable ISO to be built.

        By default, an empty list is returned.

        """
        # return []
        options = ["-b", "isolinux/isolinux.bin",
                   "-c", "isolinux/boot.cat",
                   "-no-emul-boot", "-boot-info-table",
                   "-boot-load-size", "4"]
        logging.info("144:"+isodir)
        if os.path.exists(isodir + "/isolinux/efiboot.img"):
            options.extend(["-eltorito-alt-boot",
                            "-e", "isolinux/efiboot.img",
                            "-no-emul-boot",
                            "-eltorito-alt-boot",
                            "-e", "isolinux/macboot.img",
                            "-no-emul-boot"])
        return options

    #
    # Helpers for subclasses
    #
    def _has_checkisomd5(self):
        """Check whether checkisomd5 is available in the install root."""
        def exists(instroot, path):
            return os.path.exists(instroot + path)

        if (exists(self._instroot, "/usr/lib/anaconda-runtime/checkisomd5") or
            exists(self._instroot, "/usr/bin/checkisomd5")):
            return True

        return False

    #
    # Actual implementation
    #
    def _base_on(self, base_on):
        """helper function to extract ext3 file system from a live CD ISO"""
        isoloop = DiskMount(LoopbackDisk(base_on, 0), self._mkdtemp())

        try:
            isoloop.mount()
        except MountError, e:
            raise CreatorError("Failed to loopback mount '%s' : %s" %
                               (base_on, e))

        # Copy the initrd%d.img and xen%d.gz files over to /isolinux
        # This is because the originals in /boot are removed when the
        # original .iso was created.
        src = isoloop.mountdir + "/isolinux/"
        dest = self.__ensure_isodir() + "/isolinux/"
        makedirs(dest)
        pattern = re.compile(r"(initrd\d+\.img|xen\d+\.gz)")
        files = [f for f in os.listdir(src) if pattern.search(f)
                                               and os.path.isfile(src+f)]
        for f in files:
            shutil.copyfile(src+f, dest+f)

        # legacy LiveOS filesystem layout support, remove for F9 or F10
        if os.path.exists(isoloop.mountdir + "/squashfs.img"):
            squashimg = isoloop.mountdir + "/squashfs.img"
        else:
            squashimg = isoloop.mountdir + "/LiveOS/squashfs.img"

        squashloop = DiskMount(LoopbackDisk(squashimg, 0), self._mkdtemp(), "squashfs")

        # 'self.compress_type = None' will force reading it from base_on.
        if self.compress_type is None:
            self.compress_type = squashfs_compression_type(squashimg)
            if self.compress_type == 'undetermined':
                # Default to 'gzip' for compatibility with older versions.
                self.compress_type = 'gzip'
        try:
            if not squashloop.disk.exists():
                raise CreatorError("'%s' is not a valid live CD ISO : "
                                   "squashfs.img doesn't exist" % base_on)

            try:
                squashloop.mount()
            except MountError, e:
                raise CreatorError("Failed to loopback mount squashfs.img "
                                   "from '%s' : %s" % (base_on, e))

            # legacy LiveOS filesystem layout support, remove for F9 or F10
            if os.path.exists(squashloop.mountdir + "/os.img"):
                os_image = squashloop.mountdir + "/os.img"
            else:
                os_image = squashloop.mountdir + "/LiveOS/ext3fs.img"

            if not os.path.exists(os_image):
                raise CreatorError("'%s' is not a valid live CD ISO : neither "
                                   "LiveOS/ext3fs.img nor os.img exist" %
                                   base_on)

            try:
                shutil.copyfile(os_image, self._image)
            except IOError, e:
                raise CreatorError("Failed to copy base live image to %s for modification: %s" %(self._image, e))
        finally:
            squashloop.cleanup()
            isoloop.cleanup()

    def _mount_instroot(self, base_on = None):
        self.base_on = True
        LoopImageCreator._mount_instroot(self, base_on)
        self.__write_initrd_conf(self._instroot + "/etc/sysconfig/mkinitrd")
        self.__write_dracut_conf(self._instroot + "/etc/dracut.conf.d/02livecd.conf")

    def _unmount_instroot(self):
        self.__restore_file(self._instroot + "/etc/sysconfig/mkinitrd")
        self.__restore_file(self._instroot + "/etc/dracut.conf.d/02livecd.conf")
        LoopImageCreator._unmount_instroot(self)

    def __ensure_isodir(self):
        if self.__isodir is None:
            self.__isodir = self._mkdtemp("iso-")
        return self.__isodir

    def __get_imagedir(self):
        if self.__imagedir is None:
            tempdir=self._instroot
            tempdir=tempdir[:-13]
            self.__imagedir = tempdir + "/image"
        if not os.path.exists(self.__imagedir):
            makedirs(self.__imagedir)
        return self.__imagedir

    def _generate_efiboot(self, isodir):
        """Generate EFI boot images."""
        if not glob.glob("/boot/efi/EFI/*/grubx64.efi"):
            logging.error("Missing grubx64.efi, skipping image/EFI/BOOT/grubx64.efi creation.")
            return

        if not os.path.exists(isodir + "/EFI/BOOT"):
                makedirs(isodir + "/EFI/BOOT")
        #shutil.copy("/boot/efi/EFI/ubuntu/grubx64.efi", isodir+"/EFI/BOOT/BOOTx64.EFI")
        shutil.copy("/boot/efi/EFI/ubuntu/grubx64.efi", isodir+"/EFI/BOOT/grubx64.efi")

        if not os.path.exists(isodir + "/boot/grub/"):
                makedirs(isodir + "/boot/grub/")
        bootInfo="""if loadfont /boot/grub/font.pf2 ; then
        set gfxmode=auto
        insmod efi_gop
        insmod efi_uga
        insmod gfxterm
        terminal_output gfxterm
fi

set menu_color_normal=white/black
set menu_color_highlight=black/light-gray

menuentry "Start %s 64-bit" {
        set gfxpayload=keep
        linux   /casper/vmlinuz  file=/cdrom/preseed/linuxmint.seed boot=casper iso-scan/filename=${iso_path} quiet splash --
        initrd  /casper/initrd.lz
}
menuentry "Start %s 64-bit (compatibility mode)" {
        linux   /casper/vmlinuz  file=/cdrom/preseed/linuxmint.seed boot=casper xforcevesa iso-scan/filename=${iso_path} ramdisk_size=1048576 root=/dev/ram rw noapic noapci nosplash irqpoll --
        initrd  /casper/initrd.lz
}
menuentry "Check the integrity of the medium" {
        linux   /casper/vmlinuz  boot=casper integrity-check iso-scan/filename=${iso_path} quiet splash --
        initrd  /casper/initrd.lz
}""" %(self.fslabel,self.fslabel)
        Fp = open(isodir+"/boot/grub/grub.cfg", "w")
        Fp.write(bootInfo)
        Fp.close()
        if os.path.exists(isodir+"/boot/grub/x86_64-efi"):
            shutil.rmtree(isodir+"/boot/grub/x86_64-efi", ignore_errors = True)
        shutil.copyfile("/boot/grub/fonts/unicode.pf2", isodir+"/boot/grub/font.pf2")
        shutil.copytree("/boot/grub/x86_64-efi", isodir+"/boot/grub/x86_64-efi")

        grubInfo="""insmod part_acorn
insmod part_amiga
insmod part_apple
insmod part_bsd
insmod part_dvh
insmod part_gpt
insmod part_msdos
insmod part_plan
insmod part_sun
insmod part_sunpc
configfile /boot/grub/grub.cfg"""
        Fp = open(isodir+"/boot/grub/x86_64-efi/grub.cfg", "w")
        Fp.write(grubInfo)
        Fp.close()
        return

        if not glob.glob(self._instroot+"/boot/efi/EFI/*/shim.efi"):
            logging.error("Missing shim.efi, skipping efiboot.img creation.")
            return

        # XXX-BCL: does this need --label?
        subprocess.call(["mkefiboot", isodir + "/EFI/BOOT",
                         isodir + "/isolinux/efiboot.img"])
        subprocess.call(["mkefiboot", "-a", isodir + "/EFI/BOOT",
                         isodir + "/isolinux/macboot.img", "-l", self.product,
                         "-n", "/usr/share/pixmaps/bootloader/fedora-media.vol",
                         "-i", "/usr/share/pixmaps/bootloader/fedora.icns",
                         "-p", self.product])

    def _create_bootconfig(self):
        """Configure the image so that it's bootable."""
        # self._configure_bootloader(self.__ensure_isodir())
        self._configure_bootloader(self.__get_imagedir())
        # self._generate_efiboot(self.__ensure_isodir())  #delete by zengxsh-20141009

    def _get_post_scripts_env(self, in_chroot):
        env = LoopImageCreator._get_post_scripts_env(self, in_chroot)

        if not in_chroot:
#            env["LIVE_ROOT"] = self.__ensure_isodir()
            env["LIVE_ROOT"] = self._outdir

        return env

    def __extra_filesystems(self):
        return "vfat msdos isofs ext4 xfs btrfs";

    def __extra_drivers(self):
        retval = "sr_mod sd_mod ide-cd cdrom "
        for module in self.__modules:
            if module == "=usb":
                retval = retval + "ehci_hcd uhci_hcd ohci_hcd "
                retval = retval + "usb_storage usbhid "
            elif module == "=firewire":
                retval = retval + "firewire-sbp2 firewire-ohci "
                retval = retval + "sbp2 ohci1394 ieee1394 "
            elif module == "=mmc":
                retval = retval + "mmc_block sdhci sdhci-pci "
            elif module == "=pcmcia":
                retval = retval + "pata_pcmcia "
            else:
                retval = retval + module + " "
        return retval

    def __restore_file(self,path):
        try:
            os.unlink(path)
        except:
            pass
        if os.path.exists(path + '.rpmnew'):
            os.rename(path + '.rpmnew', path)

    def __write_initrd_conf(self, path):
        if not os.path.exists(os.path.dirname(path)):
            makedirs(os.path.dirname(path))
        f = open(path, "a")
        f.write('LIVEOS="yes"\n')
        f.write('PROBE="no"\n')
        f.write('MODULES+="' + self.__extra_filesystems() + '"\n')
        f.write('MODULES+="' + self.__extra_drivers() + '"\n')
        f.close()

    def __write_dracut_conf(self, path):
        if not os.path.exists(os.path.dirname(path)):
            makedirs(os.path.dirname(path))
        f = open(path, "a")
        f.write('filesystems+="' + self.__extra_filesystems() + ' "\n')
        f.write('drivers+="' + self.__extra_drivers() + ' "\n')
        f.write('add_dracutmodules+=" dmsquash-live pollcdrom "\n')
        f.write('hostonly="no"\n')
        f.write('dracut_rescue_image="no"\n')
        f.close()

    def __create_iso(self, isodir):
        #iso = self._outdir + "/" + self.name + ".iso"
        current_time = time.strftime('%Y%m%d%H%M', time.localtime(time.time()))
        iso = self._outdir + "/" + self.name + "-" + current_time +".iso"

        logging.info("self._isofstype=%s" %(self._isofstype))
        os.chdir(self._outdir)
        os.chdir("../image")

        args = ["/usr/bin/mkisofs",
                "-r",
                "-V", self.fslabel,
                "-cache-inodes", "-J", "-l",
                "-o", iso]

        args.extend(self._get_mkisofs_options(isodir))
        if self._isofstype == "udf":
            args.append("-allow-limited-size")

        args.append(os.getcwd())
        
        if self.run_options.stop_in_mkiso == False :
            if subprocess.call(args) != 0:
                raise CreatorError("ISO creation failed!")
            logging.info("Create ISO succeed.")
            shutil.move(iso, self._outdir + "/../../")
            os.chdir(self._outdir + "/../../")
            workpre_dir = os.getcwd()
        else:
            logging.info("Skip Make ISO. run this command :")
            logging.info(" ".join(args))
            return
        if self.docleanup:
            shutil.rmtree(workpre_dir + "/work-livecd", ignore_errors = True)
        return
        # if os.path.exists("/usr/bin/isohybrid"):
        #     if os.path.exists(isodir + "/isolinux/efiboot.img"):
        #         subprocess.call(["/usr/bin/isohybrid", "-u", "-m", iso])
        #     else:
        #         subprocess.call(["/usr/bin/isohybrid", iso])

        # self.__implant_md5sum(iso)

    def __implant_md5sum(self, iso):
        """Implant an isomd5sum."""
        if os.path.exists("/usr/bin/implantisomd5"):
            implantisomd5 = "/usr/bin/implantisomd5"
        elif os.path.exists("/usr/lib/anaconda-runtime/implantisomd5"):
            implantisomd5 = "/usr/lib/anaconda-runtime/implantisomd5"
        else:
            logging.warn("isomd5sum not installed; not setting up mediacheck")
            return

        subprocess.call([implantisomd5, iso])

    def _stage_final_image(self):
        try:
            makedirs(self.__ensure_isodir() + "/LiveOS")

            #create image/casper/filesystem.manifest
            if not os.path.exists(self._instroot + "/../image/casper"):
                makedirs(self._instroot + "/../image/casper")
            cmd="sudo chroot %s dpkg-query -W --showformat='${Package} ${Version}\n'" %(self._instroot)
            pkgList=commands.getstatusoutput(cmd)
            pkgList=pkgList[1]
            Fp = open(self._instroot + "/../image/casper/filesystem.manifest", "w")
            Fp.write(pkgList)
            Fp.close()

            cmd="cp %s/../image/casper/filesystem.manifest %s/../../filesystem.manifest.log" %(self._instroot,self._instroot)
            logging.info(cmd)
            os.system(cmd)

            #create image/casper/filesystem.manifest-desktop
            cmd="cp %s/../image/casper/filesystem.manifest %s/../image/casper/filesystem.manifest-desktop" %(self._instroot,self._instroot)
            logging.info(cmd)
            os.system(cmd)

            cmd="sed -i '/ubiquity/d' %s/../image/casper/filesystem.manifest-desktop" %(self._instroot)
            logging.info(cmd)
            os.system(cmd)


            cmd="echo '\nubiquity-slideshow-cdos' >>%s/../image/casper/filesystem.manifest-desktop" %(self._instroot)
            logging.info(cmd)
            os.system(cmd)

            cmd="sed -i '/casper/d' %s/../image/casper/filesystem.manifest-desktop" %(self._instroot)
            logging.info(cmd)
            os.system(cmd)

            cmd="sed -i '/libdebian-installer/d' %s/../image/casper/filesystem.manifest-desktop" %(self._instroot)
            logging.info(cmd)
            os.system(cmd)

            cmd="sed -i '/user-setup/d' %s/../image/casper/filesystem.manifest-desktop" %(self._instroot)
            logging.info(cmd)
            os.system(cmd)




#            pkgRemove="""gir1.2-appindicator3-0.1
#gparted
#libido3-0.1-0
#python3-pam
#ubiquity
#ubiquity-casper
#ubiquity-frontend-gtk
#ubiquity-slideshow-cdos
#ubiquity-ubuntu-artwork
#ubiquity-frontend-debconf
#casper
#btrfs-tools
#libgtkmm-2.4-1v5"""
#            PkgRmFp = open(self._instroot + "/../image/casper/filesystem.manifest-remove", "w")
#            PkgRmFp.write(pkgRemove)
#            PkgRmFp.close()

            self.compress_type = "gzip"
            cmd="rm -rf %s/home/*" %(self._instroot)
            os.system(cmd)
            shutil.rmtree(self._instroot+"/debootstrap", ignore_errors = True)

            logging.info("mksquashfs start..............")
#            mksquashfs(self._instroot,
#                       self._instroot + "/../image/casper/filesystem.squashfs",
#                       self.compress_type)
            cmd="mksquashfs %s %s/../image/casper/filesystem.squashfs" %(self._instroot,self._instroot)
            logging.info(cmd)
            os.system(cmd)
            cmd="/usr/bin/du -sx --block-size=1 %s |cut -f1" %(self._instroot)
            squashfs_size=commands.getstatusoutput(cmd)
            squashfs_size=squashfs_size[1]
            sizeFp = open(self._instroot + "/../image/casper/filesystem.size", "w")
            sizeFp.write(squashfs_size)
            sizeFp.close()

            os.chdir(self._outdir)
            os.chdir("../image")
            cmd="find . -type f -print0 | xargs -0 md5sum | grep -v \"\./md5sum.txt\" |grep -v \"\./isolinux\""
            md5Value=commands.getstatusoutput(cmd)
            md5Value=md5Value[1]
            md5Fp = open(self._instroot + "/../image/md5sum.txt", "w")
            md5Fp.write(md5Value)
            md5Fp.close()

            cmd="cp %s/../image/md5sum.txt %s/../image/MD5SUMS" %(self._instroot,self._instroot)
            logging.info(cmd)
            os.system(cmd)

            logging.info("create iso start..............")
            self.__create_iso(self._instroot + "/../image")
            # self.__create_iso(self.__isodir)
            return

            self._resparse()

            if not self.skip_minimize:
                create_image_minimizer(self.__isodir + "/LiveOS/osmin.img",
                                       self._image, self.compress_type,
                                       tmpdir = self.tmpdir)

            if self.skip_compression:
                shutil.move(self._image, self.__isodir + "/LiveOS/ext3fs.img")
                if os.stat(self.__isodir + "/LiveOS/ext3fs.img").st_size >= 4*1024*1024*1024:
                    self._isofstype = "udf"
                    logging.warn("Switching to UDF due to size of LiveOS/ext3fs.img")
            else:
                makedirs(os.path.join(os.path.dirname(self._image), "LiveOS"))
                shutil.move(self._image,
                            os.path.join(os.path.dirname(self._image),
                                         "LiveOS", "ext3fs.img"))
                mksquashfs(os.path.dirname(self._image),
                           self.__isodir + "/LiveOS/squashfs.img",
                           self.compress_type)
                if os.stat(self.__isodir + "/LiveOS/squashfs.img").st_size >= 4*1024*1024*1024:
                    self._isofstype = "udf"
                    logging.warn("Switching to UDF due to size of LiveOS/squashfs.img")


            self.__create_iso(self.__isodir)
        finally:
            shutil.rmtree(self.__isodir, ignore_errors = True)
            self.__isodir = None


class x86LiveImageCreator(LiveImageCreatorBase):
    """ImageCreator for x86 machines"""
    def __init__(self, *args, **kwargs):
        LiveImageCreatorBase.__init__(self, *args, **kwargs)
        self._efiarch = None

    def _get_mkisofs_options(self, isodir):
        options = [ "-b", "isolinux/isolinux.bin",
                    "-c", "isolinux/boot.cat",
                    "-no-emul-boot", "-boot-info-table",
                    "-boot-load-size", "4" ]
        logging.info("556:"+isodir)
        if os.path.exists(isodir + "/boot/grub/efi.img"):
            options.extend([ "-eltorito-alt-boot",
                             "-e", "boot/grub/efi.img",
                             "-no-emul-boot"])

        # if os.path.exists(isodir + "/EFI/BOOT/BOOTx64.EFI"):
        #     options.extend([ "-eltorito-alt-boot",
        #                      "-e", "EFI/BOOT/BOOTx64.EFI",
        #                      "-no-emul-boot"])

        # if os.path.exists(isodir + "/isolinux/efiboot.img"):
        #     options.extend([ "-eltorito-alt-boot",
        #                      "-e", "isolinux/efiboot.img",
        #                      "-no-emul-boot",
        #                      "-eltorito-alt-boot",
        #                      "-e", "isolinux/macboot.img",
        #                      "-no-emul-boot"])
        return options


    def _get_required_packages(self):
#        return ["syslinux"] \
#               + LiveImageCreatorBase._get_required_packages(self)
         return LiveImageCreatorBase._get_required_packages(self)


    def _get_isolinux_stanzas(self, isodir):
        return ""


    def __find_syslinux_menu(self):
        for menu in ("vesamenu.c32", "menu.c32"):
#tonihu changed
            for dir in ("/usr/lib/syslinux/", "/usr/share/syslinux/"):
#            for dir in ("/usr/lib/syslinux/modules/bios", "/usr/share/syslinux/"):
                if os.path.isfile(self._instroot + dir + menu):
                    return menu

        raise CreatorError("syslinux not installed : "
                           "no suitable *menu.c32 found")


    def __find_syslinux_mboot(self):
        #
        # We only need the mboot module if we have any xen hypervisors
        #
        if not glob.glob(self._instroot + "/boot/xen.gz*"):
            return None

        return "mboot.c32"


    def __copy_syslinux_files(self, isodir, menu, mboot = None):
        files = ["isolinux.bin", "gfxboot.c32", "ldlinux.c32", "libcom32.c32", "libutil.c32", menu]
        if mboot:
            files += [mboot]

        for f in files:
#tonihu changed
            if os.path.exists(self._instroot + "/usr/lib/syslinux/" + f):
#            if os.path.exists(self._instroot + "/usr/lib/syslinux/modules/bios" + f):
                path = self._instroot + "/usr/lib/syslinux/" + f
            elif os.path.exists(self._instroot + "/usr/share/syslinux/" + f):
                path = self._instroot + "/usr/share/syslinux/" + f
            if not os.path.isfile(path):
                raise CreatorError("syslinux not installed : "
                                   "%s not found" % path)
            logging.info("copy %s %s/isolinux/" %(path,isodir))
            shutil.copy(path, isodir + "/isolinux/")


    def __copy_syslinux_background(self, isodest):
        background_path = self._instroot + \
                          "/usr/share/anaconda/boot/syslinux-vesa-splash.jpg"

        if not os.path.exists(background_path):
            # fallback to F13 location
            background_path = self._instroot + \
                              "/usr/lib/anaconda-runtime/syslinux-vesa-splash.jpg"

            if not os.path.exists(background_path):
                return False

        shutil.copyfile(background_path, isodest)

        return True


    def __copy_kernel_and_initramfs(self, isodir, version, index):
        bootdir = self._instroot + "/boot"

        shutil.copyfile(bootdir + "/vmlinuz-" + version,
                        isodir + "/isolinux/vmlinuz" + index)

        isDracut = False
        if os.path.exists(bootdir + "/initramfs-" + version + ".img"):
            shutil.copyfile(bootdir + "/initramfs-" + version + ".img",
                            isodir + "/isolinux/initrd" + index + ".img")
            isDracut = True
        elif os.path.exists(bootdir + "/initrd-" + version + ".img"):
            shutil.copyfile(bootdir + "/initrd-" + version + ".img",
                            isodir + "/isolinux/initrd" + index + ".img")
        elif not self.base_on:
            logging.error("No initrd or initramfs found for %s" % (version,))

        is_xen = False
        if os.path.exists(bootdir + "/xen.gz-" + version[:-3]):
            shutil.copyfile(bootdir + "/xen.gz-" + version[:-3],
                            isodir + "/isolinux/xen" + index + ".gz")
            is_xen = True

        return (is_xen, isDracut)


    def __is_default_kernel(self, kernel, kernels):
        if len(kernels) == 1:
            return True

        if kernel == self._default_kernel:
            return True

        if kernel.startswith("kernel-") and kernel[7:] == self._default_kernel:
            return True

        return False


    def __get_basic_syslinux_config(self, **args):
        return """
default %(menu)s
timeout %(timeout)d
menu background %(background)s
menu autoboot Starting %(title)s in # second{,s}. Press any key to interrupt.

menu clear
menu title %(title)s
menu vshift 8
menu rows 18
menu margin 8
#menu hidden
menu helpmsgrow 15
menu tabmsgrow 13

menu color border * #00000000 #00000000 none
menu color sel 0 #ffffffff #00000000 none
menu color title 0 #ff7ba3d0 #00000000 none
menu color tabmsg 0 #ff3a6496 #00000000 none
menu color unsel 0 #84b8ffff #00000000 none
menu color hotsel 0 #84b8ffff #00000000 none
menu color hotkey 0 #ffffffff #00000000 none
menu color help 0 #ffffffff #00000000 none
menu color scrollbar 0 #ffffffff #ff355594 none
menu color timeout 0 #ffffffff #00000000 none
menu color timeout_msg 0 #ffffffff #00000000 none
menu color cmdmark 0 #84b8ffff #00000000 none
menu color cmdline 0 #ffffffff #00000000 none

menu tabmsg Press Tab for full configuration options on menu items.
menu separator
""" % args


    def __get_image_stanza(self, is_xen, isDracut, **args):
        if isDracut:
            args["rootlabel"] = "live:CDLABEL=%(fslabel)s" % args
        else:
            args["rootlabel"] = "CDLABEL=%(fslabel)s" % args

        if not is_xen:
            template = """label %(short)s
  menu label %(long)s
  kernel vmlinuz%(index)s
  append initrd=initrd%(index)s.img root=%(rootlabel)s rootfstype=%(isofstype)s %(liveargs)s %(extra)s
"""
        else:
            template = """label %(short)s
  menu label %(long)s
  kernel mboot.c32
  append xen%(index)s.gz --- vmlinuz%(index)s root=%(rootlabel)s rootfstype=%(isofstype)s %(liveargs)s %(extra)s --- initrd%(index)s.img
"""
        if args.get("help"):
            template += """  text help
      %(help)s
  endtext
"""
        return template % args


    def __get_image_stanzas(self, isodir):
        kernels = self._get_kernel_versions()
        kernel_options = self._get_kernel_options()
        checkisomd5 = self._has_checkisomd5()

        # Stanzas for insertion into the config template
        linux = []
        basic = []
        check = []

        index = "0"
        for kernel, version in ((k,v) for k in kernels for v in kernels[k]):
            (is_xen, isDracut) = self.__copy_kernel_and_initramfs(isodir, version, index)
            if index == "0":
                self._isDracut = isDracut

            default = self.__is_default_kernel(kernel, kernels)

            if default:
                long = self.product
            elif kernel.startswith("kernel-"):
                long = "%s (%s)" % (self.product, kernel[7:])
            else:
                long = "%s (%s)" % (self.product, kernel)

            # tell dracut not to ask for LUKS passwords or activate mdraid sets
            if isDracut:
                kern_opts = kernel_options + " rd.luks=0 rd.md=0 rd.dm=0"
            else:
                kern_opts = kernel_options

            linux.append(self.__get_image_stanza(is_xen, isDracut,
                                           fslabel = self.fslabel,
                                           isofstype = "auto",
                                           liveargs = kern_opts,
                                           long = "^Start " + long,
                                           short = "linux" + index,
                                           extra = "",
                                           help = "",
                                           index = index))

            if default:
                linux[-1] += "  menu default\n"

            basic.append(self.__get_image_stanza(is_xen, isDracut,
                                           fslabel = self.fslabel,
                                           isofstype = "auto",
                                           liveargs = kern_opts,
                                           long = "Start " + long + " in ^basic graphics mode.",
                                           short = "basic" + index,
                                           extra = "nomodeset",
                                           help = "Try this option out if you're having trouble starting.",
                                           index = index))

            if checkisomd5:
                check.append(self.__get_image_stanza(is_xen, isDracut,
                                               fslabel = self.fslabel,
                                               isofstype = "auto",
                                               liveargs = kern_opts,
                                               long = "^Test this media & start " + long,
                                               short = "check" + index,
                                               extra = "rd.live.check",
                                               help = "",
                                               index = index))
            else:
                check.append(None)

            index = str(int(index) + 1)

        return (linux, basic, check)


    def __get_memtest_stanza(self, isodir):
        memtest = glob.glob(self._instroot + "/boot/memtest86+.bin")
        if not memtest:
            return ""

        shutil.copyfile(memtest[0], isodir + "/isolinux/memtest")

        return """label memtest
  menu label Run a ^memory test.
  text help
    If your system is having issues, an problem with your 
    system's memory may be the cause. Use this utility to 
    see if the memory is working correctly.
  endtext
  kernel memtest
"""


    def __get_local_stanza(self, isodir):
        return """label local
  menu label Boot from ^local drive
  localboot 0xffff
"""


    def _configure_auto_login(self, imageDir):
        dm_mdm_config = self._instroot+"/etc/mdm/mdm.conf"
        dm_lightdm_config = self._instroot+"/etc/lightdm"
        dm_gdm_config = self._instroot+"/etc/gdm"
        if os.path.exists(dm_mdm_config):
            mdm_auto_script="""
AutomaticLoginEnable=true
AutomaticLogin=%s
TimedLoginEnable=true
TimedLogin=%s
TimedLoginDelay=10
"""%(self.fslabel,self.fslabel)
            dm_fd=open(dm_mdm_config, "r")
            dm_content=dm_fd.read()
            dm_fd.close()
            pos=dm_content.find(r"[daemon]")
            if pos != -1:
                dm_content = dm_content[:pos+8] + mdm_auto_script[:-1] + dm_content[pos+8:]
                dm_fd=open(dm_mdm_config, "w")
                dm_fd.write(dm_content)
                dm_fd.close()
            else:
                dm_fd=open(dm_mdm_config, "a+")
                dm_fd.write("[daemon]")
                dm_fd.write(mdm_auto_script)
                dm_fd.close()
        elif os.path.exists(dm_lightdm_config):
            lightdm_auto_script="""
autologin-user=%s
autologin-user-timeout=0
"""%(self.fslabel)
            dm_config = dm_lightdm_config+"/lightdm.conf"
            if not os.path.exists(dm_config):
                dm_fd=open(dm_config, "w")
                dm_fd.write("[SeatDefaults]")
                dm_fd.write(lightdm_auto_script)
                dm_fd.close()
            else:
                dm_fd=open(dm_config, "r")
                dm_content = dm_fd.read()
                dm_fd.close()
                pos = dm_content.find(r"[SeatDefaults]")
                if pos != -1:
                    dm_content = dm_content[:pos+14] + lightdm_auto_script[:-1] + dm_content[pos+14:]
                    dm_fd=open(dm_config, "w")
                    dm_fd.write(dm_content)
                    dm_fd.close()
                else:
                    dm_fd=open(dm_config, "a+")
                    dm_fd.write("[SeatDefaults]")
                    dm_fd.write(lightdm_auto_script)
                    dm_fd.close()
        elif os.path.exists(dm_gdm_config):
# AutomaticLoginEnable=true
# AutomaticLogin=%s
# TimedLoginEnable=true
# TimedLogin=%s
# TimedLoginDelay=10
            gdm_auto_script="""
AutomaticLoginEnable=true
AutomaticLogin=%s
TimedLoginEnable=true
TimedLogin=%s
TimedLoginDelay=10
"""%(self.fslabel,self.fslabel)
            dm_config = dm_gdm_config+"/gdm.conf"
            if not os.path.exists(dm_config):
                dm_fd=open(dm_config, "w")
                dm_fd.write("[daemon]")
                dm_fd.write(gdm_auto_script)
                dm_fd.close()
            else:
                dm_fd=open(dm_config, "r")
                dm_content = dm_fd.read()
                dm_fd.close()
                pos = dm_content.find(r"[daemon]")
                if pos != -1:
                    dm_content = dm_content[:pos+8] + gdm_auto_script[:-1] + dm_content[pos+8:]
                    dm_fd=open(dm_config, "w")
                    dm_fd.write(dm_content)
                    dm_fd.close()
                else:
                    dm_fd=open(dm_config, "a+")
                    dm_fd.write("[daemon]")
                    dm_fd.write(gdm_auto_script)
                    dm_fd.close()
        else:
            raise CreatorError("not found support dm (mdm->lightdm->gdm)!")
            pass

        seed="""# Language and country.
d-i 	debian-installer/locale string zh_CN.UTF-8

# Keyboard
d-i 	console-setup/ask_detect boolean false
d-i 	console-setup/layoutcode string us

# Time
d-i 	clock-setup/ntp boolean false
d-i 	time/zone string Asia/Shanghai
d-i 	clock-setup/utc boolean false

# Don't install any task.
tasksel	tasksel/first	multiselect

# Don't install any translation packages.
d-i	pkgsel/language-pack-patterns	string

# Language support is expected to be missing.
d-i	pkgsel/install-language-support	boolean false

# Don't install language packages
d-i	pkgsel/language-packs string

# Don't show summary before install
ubiquity ubiquity/summary note
#ubiquity ubiquity/reboot boolean true"""
        preseed_dir=imageDir + "/preseed"
        if not os.path.exists(preseed_dir):
            makedirs(preseed_dir)
            LoginFp = open(preseed_dir + "/" + self.fslabel +".seed", "w")
            LoginFp.write(seed)
            LoginFp.close()


    def _configure_disk(self, imageDir):
        disk_dir=imageDir + "/.disk"
        if not os.path.exists(disk_dir):
            makedirs(disk_dir)

#        cdType="""full_cd/single"""
#        DiskFp = open(disk_dir + "/cd_type", "w")
#        DiskFp.write(cdType)
#        DiskFp.close()
#
#        cur_time = time.strftime('%Y%m%d', time.localtime(time.time()))
#        logging.info("configure_disk: arch=%s" %(self.arch))
#        DiskInfo=self.fslabel+ "    " + self.fslabel + " - Release " + self.arch + " " + cur_time
#        InfoFp = open(disk_dir + "/info", "w")
#        InfoFp.write(DiskInfo)
#        InfoFp.close()
#
#        urlFp = open(disk_dir + "/release_notes_url", "w")
#        urlFp.close()


    def _configure_copy_kernel(self, imageDir):
        casper_dir=imageDir + "/casper"
        if not os.path.exists(casper_dir):
            makedirs(casper_dir)
	#tonihu add for initrd.lz fail
        cmd="chroot " + self._instroot + " update-initramfs -u"
        os.system(cmd)
	#tonihu add end

        vmlinuzExist=False
        initrdExist=False
        latestVersion="0"
        latestVmlinuz=""
        version1=""
        boot_dir_list=os.listdir(self._instroot + "/boot")
        for filename in boot_dir_list:
            if filename.startswith("vmlinuz") or filename.startswith("kernel"):
                version1=re.sub(r"(vmlinuz-|kernel-)", "", filename)
                version1=re.sub(r".efi.signed", "", version1)
                cmd="dpkg --compare-versions " + version1 + " gt " + latestVersion
                compareResult=commands.getstatusoutput(cmd)
                if compareResult[0] == 0:
                    latestVersion=version1
                    latestVmlinuz=filename

        if latestVmlinuz.endswith(".efi.signed"):
            shutil.copyfile(self._instroot + "/boot/" + latestVmlinuz, casper_dir + "/vmlinuz")
            logging.info("Copy %s succeed." %(latestVmlinuz))
            vmlinuzExist=True
        elif os.path.isfile(self._instroot + "/boot/" + latestVmlinuz + ".efi.signed"):
            shutil.copyfile(self._instroot + "/boot/" + latestVmlinuz + ".efi.signed", casper_dir + "/vmlinuz")
            logging.info("Copy %s succeed." %(latestVmlinuz + ".efi.signed"))
            vmlinuzExist=True
        else:
            shutil.copyfile(self._instroot + "/boot/" + latestVmlinuz, casper_dir + "/vmlinuz")
            logging.info("Copy %s succeed." %(latestVmlinuz))
            vmlinuzExist=True

        initrdFileArray=["initrd.img-"+latestVersion, "initrd-"+latestVersion+".img", "initrd-"+latestVersion+".gz",\
        "initrd-"+latestVersion, "initramfs-"+latestVersion+".img"]

        for initrdFile in initrdFileArray:
            if os.path.isfile(self._instroot + "/boot/" + initrdFile):
                shutil.copyfile(self._instroot + "/boot/" + initrdFile, casper_dir + "/initrd.lz")
                logging.info("Copy %s succeed." %(initrdFile))
                initrdExist=True

        if vmlinuzExist and initrdExist:
            logging.info("Copy vmlinuz and initrd.lz succeed.")
        else:
            logging.error("Copy vmlinuz and initrd.lz failed, vmlinuz exist: %s, initrd.lz exist: %s." %(vmlinuzExist, initrdExist))


    def _get_kernel_ver(self):
        boot_dir_list=os.listdir(self._instroot + "/boot")
        for filename in boot_dir_list:
            if filename.startswith('vmlinuz'):
                ver=filename[8:]
                return ver


    def _configure_livecd_default_bootloader(self, imageDir):
        #dists/*
        if (self.arch=="amd64"):
            #boot/*
            cmd="cp -r /usr/cdspec/boot %s/" %(imageDir)
            logging.info(cmd)
            os.system(cmd)
            #EFI/*
            cmd="cp -r /usr/cdspec/EFI %s/" %(imageDir)
            logging.info(cmd)
            os.system(cmd)
        else:
            #FIXME i can't confirm for x86_x86(32bit) system is worked for this boot
            #boot
            cmd="cp -r /usr/cdspec/boot %s/" %(imageDir)
            logging.info(cmd)
            os.system(cmd)
            #EFI/*
            cmd="cp -r /usr/cdspec/EFI %s/" %(imageDir)
            logging.info(cmd)
            os.system(cmd)

        #isolinux/*
        isolinux_dir=imageDir + "/isolinux"
        if not os.path.exists(isolinux_dir):
            makedirs(isolinux_dir)

        #gpg
        cmd="cp -r /usr/cdspec/trusted.gpg %s/etc/apt/" %(self._instroot)
        logging.info(cmd)
        os.system(cmd)

        #s3g_driver_install.sh
        cmd="cp -a /usr/cdspec/s3g_driver_install.sh %s/usr/local/sbin/" %(self._instroot)
        logging.info(cmd)
        os.system(cmd)

        #nfs1.png
        cmd="cp /usr/cdspec/boot/grub/nfs1.png %s/boot/grub/" %(self._instroot)
        logging.info(cmd)
        os.system(cmd)

	#.disk
        if os.path.exists("/usr/cdspec/.disk/info"):
            os.remove("/usr/cdspec/.disk/info")
        contents = open('/tmp/work-livecd/install_root/etc/os-release').read()
        import re
        versionlist = re.findall(r'VERSION.*',contents)
        version = versionlist[0][8:].strip('"')
        info_txt="""NFS Desktop Operating System %s NFS - Release""" %(version)
        infoFp = open("/usr/cdspec/.disk/info","w+")
        infoFp.write(info_txt)
        infoFp.close()
        
        cmd="cp -r /usr/cdspec/.disk/ %s/" %(imageDir)
        logging.info(cmd)
        os.system(cmd)


	#preseed
        cmd="cp -r /usr/cdspec/preseed/ %s/" %(imageDir)
        logging.info(cmd)
        os.system(cmd)

        cmd="cp -r /usr/cdspec/isolinux/ %s/" %(imageDir)
        logging.info(cmd)
        os.system(cmd)
        self.__get_memtest_stanza(imageDir)
#tonihu changed
#        menu = self.__find_syslinux_menu()
#        self.__copy_syslinux_files(imageDir, menu, self.__find_syslinux_mboot())
#
#        langstr="""zh_CN
#"""
#        LangFp = open(isolinux_dir + "/lang", "w")
#        LangFp.write(langstr)
#        LangFp.close()
#
#        langliststr="""en
#zh_CN
#"""
#        LangListFp = open(isolinux_dir + "/langlist", "w")
#        LangListFp.write(langliststr)
#        LangListFp.close()
#
        po4astr="""[po4a_langs] zh_CN zh_TW
[po4a_paths] po/help.pot $lang:po/$lang.po
[type:docbook] help.xml
"""
        PoFp = open(isolinux_dir + "/po4a.cfg", "w")
        PoFp.write(po4astr)
        PoFp.close()
#        upper_fslabel = self.fslabel.upper()
#        txtstr="""default live
#label live
#  menu label Try and ^Install %s
#  kernel /casper/vmlinuz
#  append  file=/cdrom/preseed/%s.seed boot=casper initrd=/casper/initrd.lz quiet splash --
#label xforcevesa
#  menu label ^Install %s in compatibility mode
#  kernel /casper/vmlinuz
#  append  file=/cdrom/preseed/%s.seed boot=casper xforcevesa nomodeset b43.blacklist=yes initrd=/casper/initrd.lz ramdisk_size=1048576 root=/dev/ram rw noapic noapci nosplash irqpoll --
#label check
#  menu label ^Check disc for defects
#  kernel /casper/vmlinuz
#  append  boot=casper integrity-check initrd=/casper/initrd.lz quiet splash --
#label memtest
#  menu label Test ^memory
#  kernel memtest
#label hd
#  menu label ^Boot from first hard disk
#  localboot 0x80
#""" %(upper_fslabel,self.fslabel,upper_fslabel,self.fslabel)
#        logging.info("%s/txt.cfg \n%s" %(isolinux_dir,txtstr))
#        MenuFp = open(isolinux_dir + "/txt.cfg", "w")
#        MenuFp.write(txtstr)
#        MenuFp.close()
#
#        #boot group configure
#        cd_boot_grub_dir=imageDir + "/boot/grub"
#        boot_grub_cfg_txt = """
#if loadfont /boot/grub/font.pf2 ; then
#                set gfxmode=auto
#                insmod efi_gop
#                insmod efi_uga
#                insmod gfxterm
#                terminal_output gfxterm
#        fi
#
#        set menu_color_normal=white/black
#        set menu_color_highlight=black/light-gray
#
#        menuentry "Start %s" {
#                set gfxpayload=keep
#                linux   /casper/vmlinuz  file=/cdrom/preseed/%s.seed boot=casper iso-scan/filename=${iso_path} quiet splash --
#                initrd  /casper/initrd.lz
#        }
#        menuentry "Start %s(compatibility mode)" {
#                linux   /casper/vmlinuz  file=/cdrom/preseed/%s.seed boot=casper xforcevesa iso-scan/filename=${iso_path} ramdisk_size=1048576 root=/dev/ram rw noapic noapci nosplash irqpoll --
#                initrd  /casper/initrd.lz
#        }
#        menuentry "Check the integrity of the medium" {
#                linux   /casper/vmlinuz  boot=casper integrity-check iso-scan/filename=${iso_path} quiet splash --
#                initrd  /casper/initrd.lz
#        }
#        """%(upper_fslabel,self.fslabel,upper_fslabel,self.fslabel)
#        logging.info("%s/grup.cfg \n%s" %(isolinux_dir,boot_grub_cfg_txt))
#        MenuFp = open(cd_boot_grub_dir + "/grub.cfg", "w")
#        MenuFp.write(boot_grub_cfg_txt)
#        MenuFp.close()
#
#        boot_grub_bk_cfg_txt = """
#menuentry "Start %s" {
#             set gfxpayload=keep
#             linux     /casper/vmlinuz  file=/cdrom/preseed/%s.seed boot=casper iso-scan/filename=${iso_path} quiet splash --
#             initrd    /casper/initrd.lz
#        }
#        menuentry "Star %s (compatibility mode)" {
#             linux     /casper/vmlinuz  file=/cdrom/preseed/%s.seed boot=casper xforcevesa iso-scan/filename=${iso_path} ramdisk_size=1048576 root=/dev/ram rw noapic noapci nosplash irqpoll --
#             initrd    /casper/initrd.lz
#        }
#        menuentry "Check the integrity of the medium" {
#             linux     /casper/vmlinuz  boot=casper integrity-check iso-scan/filename=${iso_path} quiet splash --
#             initrd    /casper/initrd.lz
#        }
#        """%(upper_fslabel,self.fslabel,upper_fslabel,self.fslabel)
#        logging.info("%s/loopback.cfg \n%s" %(isolinux_dir,boot_grub_bk_cfg_txt))
#        MenuFp = open(cd_boot_grub_dir + "/loopback.cfg", "w")
#        MenuFp.write(boot_grub_bk_cfg_txt)
#        MenuFp.close()
    def _configure_livecd_create_repo(self, cdspecDIR):
        pkglist = cdspecDIR + "/pkglist"
        confDir = cdspecDIR + "/conf"
        debDir = cdspecDIR + "/deb"
        distsDir = cdspecDIR + "/dists"
        poolDir = cdspecDIR + "/pool"

        import shutil
        if os.path.exists(distsDir):
            shutil.rmtree(distsDir)
        if os.path.exists(confDir):
            shutil.rmtree(confDir)
        if os.path.exists(poolDir):
            shutil.rmtree(poolDir)

        os.mkdir(debDir)
        os.mkdir(confDir)

        distributions_txt="""
Codename:cdos
Components:main
Architectures:i386 amd64
"""
        distributionsFp = open(confDir + "/distributions","w+")
        distributionsFp.write(distributions_txt)
        distributionsFp.close()

        fp = open(pkglist)
        while True:
            line = fp.readline()
            if not line:
                break
            cmd = "cd %s;apt-get download %s" %(debDir,line)
            logging.info(cmd)
            retcode = os.system(cmd)
            if((retcode/256) != 0 ):
                raise CreatorError("download pkg error!")
        fp.close()

        cmd = "cd %s;reprepro -Vb . includedeb cdos %s/*.deb" %(cdspecDIR,debDir)
        logging.info(cmd)
        retcode = os.system(cmd)
        if((retcode/256) != 0 ):
            raise CreatorError("create iso repo error")

    def _configure_livecd_bootloader(self, imageDir):
        install_root_tmp = self._instroot + "/tmp"
        aptcdrom_deb = install_root_tmp + "/cdspec.tar.gz"
        if not os.path.exists(aptcdrom_deb):
            self._configure_livecd_create_repo("/usr/cdspec")
            cmd = "cp -rf /usr/cdspec/dists %s" %(imageDir)
            logging.info(cmd)
            os.system(cmd)

            cmd = "cp -rf /usr/cdspec/pool %s" %(imageDir)
            logging.info(cmd)
            os.system(cmd)
            self._configure_livecd_default_bootloader(imageDir)
        else:
            cmd = "tar -xzf %s -C %s/"%(aptcdrom_deb,install_root_tmp)
            logging.info(cmd)
            os.system(cmd)
            if os.path.exists(install_root_tmp+"/cdspec"):
                retcode = 0
                #dists
                inst_aptcdrom = install_root_tmp+"/cdspec"
                cmd = "cp -rf %s/dists %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files dists error!")
                #pool
                cmd = "cp -rf %s/pool %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files pool error!")
                #boot
                cmd = "cp -rf %s/boot %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files boot error!")
                #EFI
                cmd = "cp -rf %s/EFI %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files EFI error!")
                #isolinux
                cmd = "cp -rf %s/isolinux %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files isolinux error!")
                #preseed
                cmd = "cp -rf %s/preseed %s" %(inst_aptcdrom,imageDir)
                logging.info(cmd)
                retcode = os.system(cmd)
                if((retcode/256) != 0 ):
                    raise CreatorError("copy cdrom files pressed error!")

    def _configure_syslinux_bootloader(self, isodir):
        """configure the boot loader"""
        makedirs(isodir + "/isolinux")

        menu = self.__find_syslinux_menu()

        self.__copy_syslinux_files(isodir, menu,
                                   self.__find_syslinux_mboot())

        background = ""
        if self.__copy_syslinux_background(isodir + "/isolinux/splash.jpg"):
            background = "splash.jpg"

        cfg = self.__get_basic_syslinux_config(menu = menu,
                                               background = background,
                                               title = self.title,
                                               timeout = self._timeout * 10)
        cfg += "menu separator\n"

        linux, basic, check = self.__get_image_stanzas(isodir)
        # Add linux stanzas to main menu
        for s in linux:
            cfg += s
        cfg += "menu separator\n"

        cfg += """menu begin ^Troubleshooting
  menu title Troubleshooting
"""
        # Add basic video and check to submenu
        for b, c in zip(basic, check):
            cfg += b
            if c:
                cfg += c

        cfg += self.__get_memtest_stanza(isodir)
        cfg += "menu separator\n"

        cfg += self.__get_local_stanza(isodir)
        cfg += self._get_isolinux_stanzas(isodir)

        cfg += """menu separator
label returntomain
  menu label Return to ^main menu.
  menu exit
menu end
"""
        cfgf = open(isodir + "/isolinux/isolinux.cfg", "w")
        cfgf.write(cfg)
        cfgf.close()


    @property
    def efiarch(self):
        if not self._efiarch:
            # for most things, we want them named boot$efiarch
            efiarch = {"i386": "IA32", "x86_64": "X64"}
            self._efiarch = efiarch[rpmUtils.arch.getBaseArch()]
        return self._efiarch


    def __copy_efi_files(self, isodir):
        """ Copy the efi files into /EFI/BOOT/
            If any of them are missing, return False.
            requires:
              shim.efi
              gcdx64.efi
              fonts/unicode.pf2
        """
        fail = False
        missing = []
        files = [("/boot/efi/EFI/*/shim.efi", "/EFI/BOOT/BOOT%s.efi" % (self.efiarch,)),
                 ("/boot/efi/EFI/*/gcdx64.efi", "/EFI/BOOT/grubx64.efi"),
                 ("/boot/efi/EFI/*/fonts/unicode.pf2", "/EFI/BOOT/fonts/"),
                ]
        makedirs(isodir+"/EFI/BOOT/fonts/")
        for src, dest in files:
            src_glob = glob.glob(self._instroot+src)
            if not src_glob:
                missing.append("Missing EFI file (%s)" % (src,))
                fail = True
            else:
                shutil.copy(src_glob[0], isodir+dest)
        map(logging.error, missing)
        return fail


    def __get_basic_efi_config(self, **args):
        return """
set default="1"

function load_video {
  insmod efi_gop
  insmod efi_uga
  insmod video_bochs
  insmod video_cirrus
  insmod all_video
}

load_video
set gfxpayload=keep
insmod gzio
insmod part_gpt
insmod ext2

set timeout=%(timeout)d
### END /etc/grub.d/00_header ###

search --no-floppy --set=root -l '%(isolabel)s'

### BEGIN /etc/grub.d/10_linux ###
""" %args


    def __get_efi_image_stanza(self, **args):
        if self._isDracut:
            args["rootlabel"] = "live:LABEL=%(fslabel)s" % args
        else:
            args["rootlabel"] = "CDLABEL=%(fslabel)s" % args
        return """menuentry '%(long)s' --class fedora --class gnu-linux --class gnu --class os {
	linuxefi /isolinux/vmlinuz%(index)s root=%(rootlabel)s %(liveargs)s %(extra)s
	initrdefi /isolinux/initrd%(index)s.img
}
""" %args


    def __get_efi_image_stanzas(self, isodir, name):
        # FIXME: this only supports one kernel right now...

        kernel_options = self._get_kernel_options()
        checkisomd5 = self._has_checkisomd5()

        cfg = ""

        for index in range(0, 9):
            # we don't support xen kernels
            if os.path.exists("%s/EFI/BOOT/xen%d.gz" %(isodir, index)):
                continue
            cfg += self.__get_efi_image_stanza(fslabel = self.fslabel,
                                               liveargs = kernel_options,
                                               long = "Start " + self.product,
                                               extra = "", index = index)
            if checkisomd5:
                cfg += self.__get_efi_image_stanza(fslabel = self.fslabel,
                                                   liveargs = kernel_options,
                                                   long = "Test this media & start " + self.product,
                                                   extra = "rd.live.check",
                                                   index = index)
            cfg += """
submenu 'Troubleshooting -->' {
"""
            cfg += self.__get_efi_image_stanza(fslabel = self.fslabel,
                                               liveargs = kernel_options,
                                               long = "Start " + self.product + " in basic graphics mode",
                                               extra = "nomodeset", index = index)

            cfg+= """}
"""
            break

        return cfg


    def _configure_efi_bootloader(self, isodir):
        """Set up the configuration for an EFI bootloader"""
        if self.__copy_efi_files(isodir):
            shutil.rmtree(isodir + "/EFI")
            logging.warn("Failed to copy EFI files, no EFI Support will be included.")
            return

        cfg = self.__get_basic_efi_config(isolabel = self.fslabel,
                                          timeout = self._timeout)
        cfg += self.__get_efi_image_stanzas(isodir, self.name)

        cfgf = open(isodir + "/EFI/BOOT/grub.cfg", "w")
        cfgf.write(cfg)
        cfgf.close()

        # first gen mactel machines get the bootloader name wrong apparently
        if rpmUtils.arch.getBaseArch() == "i386":
            os.link(isodir + "/EFI/BOOT/BOOT%s.efi" % (self.efiarch),
                    isodir + "/EFI/BOOT/BOOT.efi")


    def _configure_bootloader(self, isodir):
#        self._configure_syslinux_bootloader(isodir)
#        self._configure_efi_bootloader(isodir)
        logging.info("call configure_livecd_bootloader %s" %(isodir))
        self._configure_livecd_bootloader(isodir)
#        logging.info("call configure_auto_login %s" %(isodir))
#        self._configure_auto_login(isodir)
        logging.info("call configure_disk %s" %(isodir))
        self._configure_disk(isodir)
        logging.info("call configure_copy_kernel %s" %(isodir))
        self._configure_copy_kernel(isodir)

class ppcLiveImageCreator(LiveImageCreatorBase):
    def _get_mkisofs_options(self, isodir):
        return [ "-hfs", "-no-desktop", "-part",
                 "-map", isodir + "/ppc/mapping",
                 "-hfs-bless", isodir + "/ppc/mac",
                 "-hfs-volid", self.fslabel ]

    def _get_required_packages(self):
        return ["yaboot"] + \
               LiveImageCreatorBase._get_required_packages(self)

    def _get_excluded_packages(self):
        # kind of hacky, but exclude memtest86+ on ppc so it can stay in cfg
        return ["memtest86+"] + \
               LiveImageCreatorBase._get_excluded_packages(self)

    def __copy_boot_file(self, destdir, file):
        for dir in [self._instroot+"/usr/share/ppc64-utils",
                    self._instroot+"/usr/lib/anaconda-runtime/boot",
                    "/usr/share/lorax/config_files/ppc"]:
            path = self._instroot + dir + "/" + file
            if not os.path.exists(path):
                continue

            makedirs(destdir)
            shutil.copy(path, destdir)
            return

        raise CreatorError("Unable to find boot file " + file)

    def __kernel_bits(self, kernel):
        testpath = (self._instroot + "/lib/modules/" +
                    kernel + "/kernel/arch/powerpc/platforms")

        if not os.path.exists(testpath):
            return { "32" : True, "64" : False }
        else:
            return { "32" : False, "64" : True }

    def __copy_kernel_and_initramfs(self, destdir, version):
        isDracut = False
        bootdir = self._instroot + "/boot"

        makedirs(destdir)

        shutil.copyfile(bootdir + "/vmlinuz-" + version,
                        destdir + "/vmlinuz")

        if os.path.exists(bootdir + "/initramfs-" + version + ".img"):
            shutil.copyfile(bootdir + "/initramfs-" + version + ".img",
                            destdir + "/initrd.img")
            isDracut = True
        else:
            shutil.copyfile(bootdir + "/initrd-" + version + ".img",
                            destdir + "/initrd.img")

        return isDracut

    def __get_basic_yaboot_config(self, **args):
        return """
init-message = "Welcome to %(name)s"
timeout=%(timeout)d
""" % args

    def __get_image_stanza(self, **args):
        if args["isDracut"]:
            args["rootlabel"] = "live:LABEL=%(fslabel)s" % args
        else:
            args["rootlabel"] = "CDLABEL=%(fslabel)s" % args
        return """

image=/ppc/ppc%(bit)s/vmlinuz
  label=%(short)s
  initrd=/ppc/ppc%(bit)s/initrd.img
  read-only
  append="root=%(rootlabel)s rootfstype=%(isofstype)s %(liveargs)s %(extra)s"
""" % args


    def __write_yaboot_config(self, isodir, bit, isDracut = False):
        cfg = self.__get_basic_yaboot_config(name = self.name,
                                             timeout = self._timeout * 100)

        kernel_options = self._get_kernel_options()

        cfg += self.__get_image_stanza(fslabel = self.fslabel,
                                       isofstype = "auto",
                                       short = "linux",
                                       long = "Run from image",
                                       extra = "",
                                       bit = bit,
                                       liveargs = kernel_options,
                                       isDracut = isDracut)

        if self._has_checkisomd5():
            cfg += self.__get_image_stanza(fslabel = self.fslabel,
                                           isofstype = "auto",
                                           short = "rd.live.check",
                                           long = "Verify and run from image",
                                           extra = "rd.live.check",
                                           bit = bit,
                                           liveargs = kernel_options,
                                           isDracut = isDracut)

        f = open(isodir + "/ppc/ppc" + bit + "/yaboot.conf", "w")
        f.write(cfg)
        f.close()

    def __write_not_supported(self, isodir, bit):
        makedirs(isodir + "/ppc/ppc" + bit)

        message = "Sorry, this LiveCD does not support your hardware"

        f = open(isodir + "/ppc/ppc" + bit + "/yaboot.conf", "w")
        f.write('init-message = "' + message + '"')
        f.close()


    def __write_dualbits_yaboot_config(isodir, **args):
        cfg = """
init-message = "\nWelcome to %(name)s!\nUse 'linux32' for 32-bit kernel.\n\n"
timeout=%(timeout)d
default=linux

image=/ppc/ppc64/vmlinuz
	label=linux64
	alias=linux
	initrd=/ppc/ppc64/initrd.img
	read-only

image=/ppc/ppc32/vmlinuz
	label=linux32
	initrd=/ppc/ppc32/initrd.img
	read-only
""" % args

        f = open(isodir + "/etc/yaboot.conf", "w")
        f.write(cfg)
        f.close()

    def _configure_bootloader(self, isodir):
        """configure the boot loader"""
        havekernel = { 32: False, 64: False }

        self.__copy_boot_file(isodir + "/ppc", "mapping")
        self.__copy_boot_file(isodir + "/ppc", "bootinfo.txt")
        self.__copy_boot_file(isodir + "/ppc/mac", "ofboot.b")

        shutil.copyfile(self._instroot + "/usr/lib/yaboot/yaboot",
                        isodir + "/ppc/mac/yaboot")

        makedirs(isodir + "/ppc/chrp")
        shutil.copyfile(self._instroot + "/usr/lib/yaboot/yaboot",
                        isodir + "/ppc/chrp/yaboot")

        subprocess.call(["/usr/sbin/addnote", isodir + "/ppc/chrp/yaboot"])

        #
        # FIXME: ppc should support multiple kernels too...
        #
        kernel = self._get_kernel_versions().values()[0][0]

        kernel_bits = self.__kernel_bits(kernel)

        for (bit, present) in kernel_bits.items():
            if not present:
                self.__write_not_supported(isodir, bit)
                continue

            isDracut = self.__copy_kernel_and_initramfs(isodir + "/ppc/ppc" + bit, kernel)
            self.__write_yaboot_config(isodir, bit, isDracut)

        makedirs(isodir + "/etc")
        if kernel_bits["32"] and not kernel_bits["64"]:
            shutil.copyfile(isodir + "/ppc/ppc32/yaboot.conf",
                            isodir + "/etc/yaboot.conf")
        elif kernel_bits["64"] and not kernel_bits["32"]:
            shutil.copyfile(isodir + "/ppc/ppc64/yaboot.conf",
                            isodir + "/etc/yaboot.conf")
        else:
            self.__write_dualbits_yaboot_config(isodir,
                                                name = self.name,
                                                timeout = self._timeout * 100)

        #
        # FIXME: build 'netboot' images with kernel+initrd, like mk-images.ppc
        #

class ppc64LiveImageCreator(ppcLiveImageCreator):
    def _get_excluded_packages(self):
        # FIXME:
        #   while kernel.ppc and kernel.ppc64 co-exist,
        #   we can't have both
        return ["kernel.ppc"] + \
               ppcLiveImageCreator._get_excluded_packages(self)

#arch = rpmUtils.arch.getBaseArch()
arch = "i386"   #zengxsh,2014.8.28
if arch in ("i386", "x86_64", "amd64"):
    LiveImageCreator = x86LiveImageCreator
elif arch in ("ppc",):
    LiveImageCreator = ppcLiveImageCreator
elif arch in ("ppc64",):
    LiveImageCreator = ppc64LiveImageCreator
elif arch.startswith('arm'):
    LiveImageCreator = LiveImageCreatorBase

else:
    raise CreatorError("Architecture not supported!")
